/*
 * Copyright (C) 2016 Allwinnertech
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Adjustable factor-based clock implementation
 */

#include "clk-sun8iw15.h"

/*
 * freq table from hardware, need follow rules
 * 1)   each table  named as
 *      factor_pll1_tbl
 *      factor_pll2_tbl
 *      ...
 * 2) for each table line
 *      a) follow the format PLLx(n, k, m, p, d1, d2, freq), and keep the
 *         factors order
 *      b) if any factor not used, skip it
 *      c) the factor is the value to write registers, not means factor + 1
 *
 *      example
 *      PLL1(9, 0, 0, 2, 60000000) means PLL1(n, k, m, p, freq)
 *      PLLVIDEO0(3, 0, 96000000) means PLLVIDEO0(n, m, freq)
 *
 */

/* PLLCPU(n, m, p, freq)	F_N8X8_M0X2_P16x2 */
struct sunxi_clk_factor_freq factor_pllcpu_tbl[] = {
PLLCPU(11,     2,     72000000U),
PLLCPU(31,     2,     192000000U),
PLLCPU(11,     0,     288000000U),
PLLCPU(15,     0,     384000000U),
PLLCPU(40,     1,     492000000U),
PLLCPU(24,     0,     600000000U),
PLLCPU(64,     1,     780000000U),
PLLCPU(72,     1,     876000000U),
PLLCPU(80,     1,     972000000U),
PLLCPU(44,     0,     1080000000U),
PLLCPU(46,     0,     1128000000U),
PLLCPU(48,     0,     1176000000U),
PLLCPU(49,     0,     1200000000U),
PLLCPU(51,     0,     1248000000U),
PLLCPU(53,     0,     1296000000U),
PLLCPU(54,     0,     1320000000U),
PLLCPU(58,     0,     1416000000U),
PLLCPU(122,    1,     1476000000U),
PLLCPU(124,    1,     1500000000U),
PLLCPU(62,     0,     1512000000U),
PLLCPU(63,     0,     1536000000U),
PLLCPU(132,    1,     1596000000U),
PLLCPU(134,    1,     1620000000U),
PLLCPU(138,    1,     1668000000U),
PLLCPU(73,     0,     1776000000U),
PLLCPU(74,     0,     1800000000U),
PLLCPU(76,     0,     1848000000U),
PLLCPU(160,    1,     1932000000U),
PLLCPU(81,     0,     1968000000U),
PLLCPU(166,    1,     2004000000U),
PLLCPU(84,     0,     2040000000U),
};
/*n  m1   m0   freq */
/* PLLDDR(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllddr_tbl[] = {
PLLDDR(23,     0,     1,     288000000U),
PLLDDR(29,     0,     1,     360000000U),
PLLDDR(18,     0,     0,     456000000U),
PLLDDR(90,     1,     1,     546000000U),
PLLDDR(24,     0,     0,     600000000U),
PLLDDR(58,     0,     1,     708000000U),
PLLDDR(34,     0,     0,     840000000U),
PLLDDR(39,     0,     0,     960000000U),
PLLDDR(44,     0,     0,     1080000000U),
PLLDDR(84,     0,     0,     2040000000U),
};

/* PLLPERIPH0(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllperiph0_tbl[] = {
PLLPERIPH0(49,     1,     1,     300000000U),
PLLPERIPH0(56,     1,     1,     342000000U),
PLLPERIPH0(31,     0,     1,     384000000U),
PLLPERIPH0(19,     0,     0,     480000000U),
PLLPERIPH0(23,     0,     0,     576000000U),
PLLPERIPH0(24,     0,     0,     600000000U),
PLLPERIPH0(25,     0,     0,     624000000U),
PLLPERIPH0(52,     0,     1,     636000000U),
};

/* PLLPERIPH1(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllperiph1_tbl[] = {
PLLPERIPH1(49,     1,     1,     300000000U),
PLLPERIPH1(56,     1,     1,     342000000U),
PLLPERIPH1(31,     0,     1,     384000000U),
PLLPERIPH1(19,     0,     0,     480000000U),
PLLPERIPH1(23,     0,     0,     576000000U),
PLLPERIPH1(24,     0,     0,     600000000U),
PLLPERIPH1(25,     0,     0,     624000000U),
PLLPERIPH1(52,     0,     1,     636000000U),
};

/* PLLGPU(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllgpu_tbl[] = {
PLLGPU(23,     0,     1,     288000000U),
PLLGPU(24,     0,     1,     300000000U),
PLLGPU(25,     0,     1,     312000000U),
PLLGPU(26,     0,     1,     324000000U),
PLLGPU(27,     0,     1,     336000000U),
PLLGPU(28,     0,     1,     348000000U),
PLLGPU(29,     0,     1,     360000000U),
PLLGPU(30,     0,     1,     372000000U),
PLLGPU(15,     0,     0,     384000000U),
PLLGPU(32,     0,     1,     396000000U),
PLLGPU(16,     0,     0,     408000000U),
PLLGPU(34,     0,     1,     420000000U),
PLLGPU(17,     0,     0,     432000000U),
PLLGPU(36,     0,     1,     444000000U),
PLLGPU(18,     0,     0,     456000000U),
PLLGPU(38,     0,     1,     468000000U),
PLLGPU(19,     0,     0,     480000000U),
PLLGPU(40,     0,     1,     492000000U),
PLLGPU(20,     0,     0,     504000000U),
PLLGPU(42,     0,     1,     516000000U),
PLLGPU(21,     0,     0,     528000000U),
PLLGPU(44,     0,     1,     540000000U),
PLLGPU(22,     0,     0,     552000000U),
PLLGPU(46,     0,     1,     564000000U),
PLLGPU(23,     0,     0,     576000000U),
PLLGPU(48,     0,     1,     588000000U),
PLLGPU(24,     0,     0,     600000000U),
PLLGPU(50,     0,     1,     612000000U),
PLLGPU(25,     0,     0,     624000000U),
PLLGPU(52,     0,     1,     636000000U),
PLLGPU(26,     0,     0,     648000000U),
PLLGPU(54,     0,     1,     660000000U),
PLLGPU(27,     0,     0,     672000000U),
};

/* PLLVIDEO0(n, d1, freq)	F_N8X8_D1V1X1 */
struct sunxi_clk_factor_freq factor_pllvideo0_tbl[] = {
PLLVIDEO0(23,     1,     72000000U),
PLLVIDEO0(24,     1,     75000000U),
PLLVIDEO0(25,     1,     78000000U),
PLLVIDEO0(26,     1,     81000000U),
PLLVIDEO0(13,     0,     84000000U),
PLLVIDEO0(28,     1,     87000000U),
PLLVIDEO0(14,     0,     90000000U),
PLLVIDEO0(30,     1,     93000000U),
PLLVIDEO0(15,     0,     96000000U),
PLLVIDEO0(32,     1,     99000000U),
PLLVIDEO0(16,     0,     102000000U),
PLLVIDEO0(34,     1,     105000000U),
PLLVIDEO0(17,     0,     108000000U),
PLLVIDEO0(36,     1,     111000000U),
PLLVIDEO0(18,     0,     114000000U),
PLLVIDEO0(38,     1,     117000000U),
PLLVIDEO0(19,     0,     120000000U),
PLLVIDEO0(40,     1,     123000000U),
PLLVIDEO0(20,     0,     126000000U),
PLLVIDEO0(42,     1,     129000000U),
PLLVIDEO0(21,     0,     132000000U),
PLLVIDEO0(44,     1,     135000000U),
PLLVIDEO0(22,     0,     138000000U),
PLLVIDEO0(46,     1,     141000000U),
PLLVIDEO0(23,     0,     144000000U),
PLLVIDEO0(48,     1,     147000000U),
PLLVIDEO0(24,     0,     150000000U),
PLLVIDEO0(50,     1,     153000000U),
PLLVIDEO0(25,     0,     156000000U),
PLLVIDEO0(52,     1,     159000000U),
PLLVIDEO0(26,     0,     162000000U),
PLLVIDEO0(54,     1,     165000000U),
PLLVIDEO0(27,     0,     168000000U),
PLLVIDEO0(56,     1,     171000000U),
PLLVIDEO0(28,     0,     174000000U),
PLLVIDEO0(58,     1,     177000000U),
PLLVIDEO0(29,     0,     180000000U),
PLLVIDEO0(60,     1,     183000000U),
PLLVIDEO0(30,     0,     186000000U),
PLLVIDEO0(62,     1,     189000000U),
PLLVIDEO0(31,     0,     192000000U),
PLLVIDEO0(64,     1,     195000000U),
PLLVIDEO0(32,     0,     198000000U),
PLLVIDEO0(66,     1,     201000000U),
PLLVIDEO0(33,     0,     204000000U),
PLLVIDEO0(68,     1,     207000000U),
PLLVIDEO0(34,     0,     210000000U),
PLLVIDEO0(70,     1,     213000000U),
PLLVIDEO0(35,     0,     216000000U),
PLLVIDEO0(72,     1,     219000000U),
PLLVIDEO0(36,     0,     222000000U),
PLLVIDEO0(74,     1,     225000000U),
PLLVIDEO0(37,     0,     228000000U),
PLLVIDEO0(76,     1,     231000000U),
PLLVIDEO0(38,     0,     234000000U),
PLLVIDEO0(78,     1,     237000000U),
PLLVIDEO0(39,     0,     240000000U),
PLLVIDEO0(80,     1,     243000000U),
PLLVIDEO0(40,     0,     246000000U),
PLLVIDEO0(82,     1,     249000000U),
PLLVIDEO0(41,     0,     252000000U),
PLLVIDEO0(84,     1,     255000000U),
PLLVIDEO0(42,     0,     258000000U),
PLLVIDEO0(86,     1,     261000000U),
PLLVIDEO0(43,     0,     264000000U),
PLLVIDEO0(88,     1,     267000000U),
PLLVIDEO0(44,     0,     270000000U),
PLLVIDEO0(90,     1,     273000000U),
PLLVIDEO0(45,     0,     276000000U),
PLLVIDEO0(92,     1,     279000000U),
PLLVIDEO0(46,     0,     282000000U),
PLLVIDEO0(94,     1,     285000000U),
PLLVIDEO0(47,     0,     288000000U),
PLLVIDEO0(96,     1,     291000000U),
PLLVIDEO0(48,     0,     294000000U),
PLLVIDEO0(98,     1,     297000000U),
PLLVIDEO0(49,     0,     300000000U),
PLLVIDEO0(100,     1,     303000000U),
PLLVIDEO0(50,     0,     306000000U),
PLLVIDEO0(102,     1,     309000000U),
PLLVIDEO0(51,     0,     312000000U),
PLLVIDEO0(104,     1,     315000000U),
PLLVIDEO0(52,     0,     318000000U),
PLLVIDEO0(106,     1,     321000000U),
PLLVIDEO0(53,     0,     324000000U),
PLLVIDEO0(108,     1,     327000000U),
PLLVIDEO0(54,     0,     330000000U),
PLLVIDEO0(110,     1,     333000000U),
PLLVIDEO0(55,     0,     336000000U),
PLLVIDEO0(112,     1,     339000000U),
PLLVIDEO0(56,     0,     342000000U),
PLLVIDEO0(114,     1,     345000000U),
PLLVIDEO0(57,     0,     348000000U),
PLLVIDEO0(116,     1,     351000000U),
PLLVIDEO0(58,     0,     354000000U),
PLLVIDEO0(118,     1,     357000000U),
PLLVIDEO0(59,     0,     360000000U),
PLLVIDEO0(120,     1,     363000000U),
PLLVIDEO0(60,     0,     366000000U),
PLLVIDEO0(122,     1,     369000000U),
PLLVIDEO0(61,     0,     372000000U),
PLLVIDEO0(124,     1,     375000000U),
PLLVIDEO0(62,     0,     378000000U),
PLLVIDEO0(126,     1,     381000000U),
PLLVIDEO0(63,     0,     384000000U),
PLLVIDEO0(128,     1,     387000000U),
PLLVIDEO0(64,     0,     390000000U),
PLLVIDEO0(130,     1,     393000000U),
PLLVIDEO0(65,     0,     396000000U),
PLLVIDEO0(132,     1,     399000000U),
PLLVIDEO0(66,     0,     402000000U),
PLLVIDEO0(134,     1,     405000000U),
PLLVIDEO0(67,     0,     408000000U),
PLLVIDEO0(136,     1,     411000000U),
PLLVIDEO0(68,     0,     414000000U),
PLLVIDEO0(138,     1,     417000000U),
PLLVIDEO0(69,     0,     420000000U),
PLLVIDEO0(140,     1,     423000000U),
PLLVIDEO0(70,     0,     426000000U),
PLLVIDEO0(142,     1,     429000000U),
PLLVIDEO0(71,     0,     432000000U),
PLLVIDEO0(144,     1,     435000000U),
PLLVIDEO0(72,     0,     438000000U),
PLLVIDEO0(146,     1,     441000000U),
PLLVIDEO0(73,     0,     444000000U),
PLLVIDEO0(148,     1,     447000000U),
PLLVIDEO0(74,     0,     450000000U),
PLLVIDEO0(150,     1,     453000000U),
PLLVIDEO0(75,     0,     456000000U),
PLLVIDEO0(152,     1,     459000000U),
PLLVIDEO0(76,     0,     462000000U),
PLLVIDEO0(154,     1,     465000000U),
PLLVIDEO0(77,     0,     468000000U),
PLLVIDEO0(156,     1,     471000000U),
PLLVIDEO0(78,     0,     474000000U),
PLLVIDEO0(158,     1,     477000000U),
PLLVIDEO0(79,     0,     480000000U),
PLLVIDEO0(160,     1,     483000000U),
PLLVIDEO0(80,     0,     486000000U),
PLLVIDEO0(162,     1,     489000000U),
PLLVIDEO0(81,     0,     492000000U),
PLLVIDEO0(164,     1,     495000000U),
PLLVIDEO0(82,     0,     498000000U),
PLLVIDEO0(166,     1,     501000000U),
PLLVIDEO0(83,     0,     504000000U),
PLLVIDEO0(168,     1,     507000000U),
PLLVIDEO0(84,     0,     510000000U),
PLLVIDEO0(170,     1,     513000000U),
PLLVIDEO0(85,     0,     516000000U),
PLLVIDEO0(172,     1,     519000000U),
PLLVIDEO0(86,     0,     522000000U),
PLLVIDEO0(174,     1,     525000000U),
PLLVIDEO0(87,     0,     528000000U),
PLLVIDEO0(176,     1,     531000000U),
PLLVIDEO0(88,     0,     534000000U),
PLLVIDEO0(178,     1,     537000000U),
PLLVIDEO0(89,     0,     540000000U),
};

/* PLLVIDEO1(n, d1, freq)	F_N8X8_D1V1X1 */
struct sunxi_clk_factor_freq factor_pllvideo1_tbl[] = {
PLLVIDEO1(23,     1,     72000000U),
PLLVIDEO1(24,     1,     75000000U),
PLLVIDEO1(25,     1,     78000000U),
PLLVIDEO1(26,     1,     81000000U),
PLLVIDEO1(13,     0,     84000000U),
PLLVIDEO1(28,     1,     87000000U),
PLLVIDEO1(14,     0,     90000000U),
PLLVIDEO1(30,     1,     93000000U),
PLLVIDEO1(15,     0,     96000000U),
PLLVIDEO1(32,     1,     99000000U),
PLLVIDEO1(16,     0,     102000000U),
PLLVIDEO1(34,     1,     105000000U),
PLLVIDEO1(17,     0,     108000000U),
PLLVIDEO1(36,     1,     111000000U),
PLLVIDEO1(18,     0,     114000000U),
PLLVIDEO1(38,     1,     117000000U),
PLLVIDEO1(19,     0,     120000000U),
PLLVIDEO1(40,     1,     123000000U),
PLLVIDEO1(20,     0,     126000000U),
PLLVIDEO1(42,     1,     129000000U),
PLLVIDEO1(21,     0,     132000000U),
PLLVIDEO1(44,     1,     135000000U),
PLLVIDEO1(22,     0,     138000000U),
PLLVIDEO1(46,     1,     141000000U),
PLLVIDEO1(23,     0,     144000000U),
PLLVIDEO1(48,     1,     147000000U),
PLLVIDEO1(24,     0,     150000000U),
PLLVIDEO1(50,     1,     153000000U),
PLLVIDEO1(25,     0,     156000000U),
PLLVIDEO1(52,     1,     159000000U),
PLLVIDEO1(26,     0,     162000000U),
PLLVIDEO1(54,     1,     165000000U),
PLLVIDEO1(27,     0,     168000000U),
PLLVIDEO1(56,     1,     171000000U),
PLLVIDEO1(28,     0,     174000000U),
PLLVIDEO1(58,     1,     177000000U),
PLLVIDEO1(29,     0,     180000000U),
PLLVIDEO1(60,     1,     183000000U),
PLLVIDEO1(30,     0,     186000000U),
PLLVIDEO1(62,     1,     189000000U),
PLLVIDEO1(31,     0,     192000000U),
PLLVIDEO1(64,     1,     195000000U),
PLLVIDEO1(32,     0,     198000000U),
PLLVIDEO1(66,     1,     201000000U),
PLLVIDEO1(33,     0,     204000000U),
PLLVIDEO1(68,     1,     207000000U),
PLLVIDEO1(34,     0,     210000000U),
PLLVIDEO1(70,     1,     213000000U),
PLLVIDEO1(35,     0,     216000000U),
PLLVIDEO1(72,     1,     219000000U),
PLLVIDEO1(36,     0,     222000000U),
PLLVIDEO1(74,     1,     225000000U),
PLLVIDEO1(37,     0,     228000000U),
PLLVIDEO1(76,     1,     231000000U),
PLLVIDEO1(38,     0,     234000000U),
PLLVIDEO1(78,     1,     237000000U),
PLLVIDEO1(39,     0,     240000000U),
PLLVIDEO1(80,     1,     243000000U),
PLLVIDEO1(40,     0,     246000000U),
PLLVIDEO1(82,     1,     249000000U),
PLLVIDEO1(41,     0,     252000000U),
PLLVIDEO1(84,     1,     255000000U),
PLLVIDEO1(42,     0,     258000000U),
PLLVIDEO1(86,     1,     261000000U),
PLLVIDEO1(43,     0,     264000000U),
PLLVIDEO1(88,     1,     267000000U),
PLLVIDEO1(44,     0,     270000000U),
PLLVIDEO1(90,     1,     273000000U),
PLLVIDEO1(45,     0,     276000000U),
PLLVIDEO1(92,     1,     279000000U),
PLLVIDEO1(46,     0,     282000000U),
PLLVIDEO1(94,     1,     285000000U),
PLLVIDEO1(47,     0,     288000000U),
PLLVIDEO1(96,     1,     291000000U),
PLLVIDEO1(48,     0,     294000000U),
PLLVIDEO1(98,     1,     297000000U),
PLLVIDEO1(49,     0,     300000000U),
PLLVIDEO1(100,     1,     303000000U),
PLLVIDEO1(50,     0,     306000000U),
PLLVIDEO1(102,     1,     309000000U),
PLLVIDEO1(51,     0,     312000000U),
PLLVIDEO1(104,     1,     315000000U),
PLLVIDEO1(52,     0,     318000000U),
PLLVIDEO1(106,     1,     321000000U),
PLLVIDEO1(53,     0,     324000000U),
PLLVIDEO1(108,     1,     327000000U),
PLLVIDEO1(54,     0,     330000000U),
PLLVIDEO1(110,     1,     333000000U),
PLLVIDEO1(55,     0,     336000000U),
PLLVIDEO1(112,     1,     339000000U),
PLLVIDEO1(56,     0,     342000000U),
PLLVIDEO1(114,     1,     345000000U),
PLLVIDEO1(57,     0,     348000000U),
PLLVIDEO1(116,     1,     351000000U),
PLLVIDEO1(58,     0,     354000000U),
PLLVIDEO1(118,     1,     357000000U),
PLLVIDEO1(59,     0,     360000000U),
PLLVIDEO1(120,     1,     363000000U),
PLLVIDEO1(60,     0,     366000000U),
PLLVIDEO1(122,     1,     369000000U),
PLLVIDEO1(61,     0,     372000000U),
PLLVIDEO1(124,     1,     375000000U),
PLLVIDEO1(62,     0,     378000000U),
PLLVIDEO1(126,     1,     381000000U),
PLLVIDEO1(63,     0,     384000000U),
PLLVIDEO1(128,     1,     387000000U),
PLLVIDEO1(64,     0,     390000000U),
PLLVIDEO1(130,     1,     393000000U),
PLLVIDEO1(65,     0,     396000000U),
PLLVIDEO1(132,     1,     399000000U),
PLLVIDEO1(66,     0,     402000000U),
PLLVIDEO1(134,     1,     405000000U),
PLLVIDEO1(67,     0,     408000000U),
PLLVIDEO1(136,     1,     411000000U),
PLLVIDEO1(68,     0,     414000000U),
PLLVIDEO1(138,     1,     417000000U),
PLLVIDEO1(69,     0,     420000000U),
PLLVIDEO1(140,     1,     423000000U),
PLLVIDEO1(70,     0,     426000000U),
PLLVIDEO1(142,     1,     429000000U),
PLLVIDEO1(71,     0,     432000000U),
PLLVIDEO1(144,     1,     435000000U),
PLLVIDEO1(72,     0,     438000000U),
PLLVIDEO1(146,     1,     441000000U),
PLLVIDEO1(73,     0,     444000000U),
PLLVIDEO1(148,     1,     447000000U),
PLLVIDEO1(74,     0,     450000000U),
PLLVIDEO1(150,     1,     453000000U),
PLLVIDEO1(75,     0,     456000000U),
PLLVIDEO1(152,     1,     459000000U),
PLLVIDEO1(76,     0,     462000000U),
PLLVIDEO1(154,     1,     465000000U),
PLLVIDEO1(77,     0,     468000000U),
PLLVIDEO1(156,     1,     471000000U),
PLLVIDEO1(78,     0,     474000000U),
PLLVIDEO1(158,     1,     477000000U),
PLLVIDEO1(79,     0,     480000000U),
PLLVIDEO1(160,     1,     483000000U),
PLLVIDEO1(80,     0,     486000000U),
PLLVIDEO1(162,     1,     489000000U),
PLLVIDEO1(81,     0,     492000000U),
PLLVIDEO1(164,     1,     495000000U),
PLLVIDEO1(82,     0,     498000000U),
PLLVIDEO1(166,     1,     501000000U),
PLLVIDEO1(83,     0,     504000000U),
PLLVIDEO1(168,     1,     507000000U),
PLLVIDEO1(84,     0,     510000000U),
PLLVIDEO1(170,     1,     513000000U),
PLLVIDEO1(85,     0,     516000000U),
PLLVIDEO1(172,     1,     519000000U),
PLLVIDEO1(86,     0,     522000000U),
PLLVIDEO1(174,     1,     525000000U),
PLLVIDEO1(87,     0,     528000000U),
PLLVIDEO1(176,     1,     531000000U),
PLLVIDEO1(88,     0,     534000000U),
PLLVIDEO1(178,     1,     537000000U),
PLLVIDEO1(89,     0,     540000000U),
};

/* PLLVE(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllve_tbl[] = {
PLLVE(23,     0,     1,     288000000U),
PLLVE(24,     0,     1,     300000000U),
PLLVE(25,     0,     1,     312000000U),
PLLVE(26,     0,     1,     324000000U),
PLLVE(27,     0,     1,     336000000U),
PLLVE(28,     0,     1,     348000000U),
PLLVE(29,     0,     1,     360000000U),
PLLVE(30,     0,     1,     372000000U),
PLLVE(15,     0,     0,     384000000U),
PLLVE(32,     0,     1,     396000000U),
PLLVE(16,     0,     0,     408000000U),
PLLVE(34,     0,     1,     420000000U),
PLLVE(17,     0,     0,     432000000U),
PLLVE(36,     0,     1,     444000000U),
PLLVE(18,     0,     0,     456000000U),
PLLVE(38,     0,     1,     468000000U),
PLLVE(19,     0,     0,     480000000U),
PLLVE(40,     0,     1,     492000000U),
PLLVE(20,     0,     0,     504000000U),
PLLVE(42,     0,     1,     516000000U),
PLLVE(21,     0,     0,     528000000U),
PLLVE(44,     0,     1,     540000000U),
PLLVE(22,     0,     0,     552000000U),
PLLVE(46,     0,     1,     564000000U),
PLLVE(23,     0,     0,     576000000U),
PLLVE(48,     0,     1,     588000000U),
PLLVE(24,     0,     0,     600000000U),
PLLVE(50,     0,     1,     612000000U),
PLLVE(25,     0,     0,     624000000U),
PLLVE(52,     0,     1,     636000000U),
PLLVE(26,     0,     0,     648000000U),
PLLVE(54,     0,     1,     660000000U),
PLLVE(27,     0,     0,     672000000U),
PLLVE(56,     0,     1,     684000000U),
PLLVE(28,     0,     0,     696000000U),
PLLVE(58,     0,     1,     708000000U),
PLLVE(29,     0,     0,     720000000U),
PLLVE(60,     0,     1,     732000000U),
PLLVE(30,     0,     0,     744000000U),
PLLVE(62,     0,     1,     756000000U),
PLLVE(31,     0,     0,     768000000U),
PLLVE(64,     0,     1,     780000000U),
PLLVE(32,     0,     0,     792000000U),
PLLVE(66,     0,     1,     804000000U),
PLLVE(33,     0,     0,     816000000U),
PLLVE(68,     0,     1,     828000000U),
PLLVE(34,     0,     0,     840000000U),
PLLVE(70,     0,     1,     852000000U),
PLLVE(35,     0,     0,     864000000U),
PLLVE(72,     0,     1,     876000000U),
PLLVE(36,     0,     0,     888000000U),
PLLVE(74,     0,     1,     900000000U),
PLLVE(37,     0,     0,     912000000U),
PLLVE(76,     0,     1,     924000000U),
PLLVE(38,     0,     0,     936000000U),
PLLVE(78,     0,     1,     948000000U),
PLLVE(39,     0,     0,     960000000U),
PLLVE(80,     0,     1,     972000000U),
PLLVE(40,     0,     0,     984000000U),
PLLVE(82,     0,     1,     996000000U),
PLLVE(41,     0,     0,     1008000000U),
};

/* PLLDE(n, d1, d2, freq)	F_N8X8_D1V1X1_D2V0X1 */
struct sunxi_clk_factor_freq factor_pllde_tbl[] = {
PLLDE(23,     0,     1,     288000000U),
PLLDE(24,     0,     1,     300000000U),
PLLDE(25,     0,     1,     312000000U),
PLLDE(26,     0,     1,     324000000U),
PLLDE(27,     0,     1,     336000000U),
PLLDE(28,     0,     1,     348000000U),
PLLDE(29,     0,     1,     360000000U),
PLLDE(30,     0,     1,     372000000U),
PLLDE(15,     0,     0,     384000000U),
PLLDE(32,     0,     1,     396000000U),
PLLDE(16,     0,     0,     408000000U),
PLLDE(34,     0,     1,     420000000U),
PLLDE(17,     0,     0,     432000000U),
PLLDE(36,     0,     1,     444000000U),
PLLDE(18,     0,     0,     456000000U),
PLLDE(38,     0,     1,     468000000U),
PLLDE(19,     0,     0,     480000000U),
PLLDE(40,     0,     1,     492000000U),
PLLDE(20,     0,     0,     504000000U),
PLLDE(42,     0,     1,     516000000U),
PLLDE(21,     0,     0,     528000000U),
PLLDE(44,     0,     1,     540000000U),
PLLDE(22,     0,     0,     552000000U),
PLLDE(46,     0,     1,     564000000U),
PLLDE(23,     0,     0,     576000000U),
PLLDE(48,     0,     1,     588000000U),
PLLDE(24,     0,     0,     600000000U),
PLLDE(50,     0,     1,     612000000U),
PLLDE(25,     0,     0,     624000000U),
PLLDE(52,     0,     1,     636000000U),
PLLDE(26,     0,     0,     648000000U),
PLLDE(54,     0,     1,     660000000U),
PLLDE(27,     0,     0,     672000000U),
PLLDE(56,     0,     1,     684000000U),
PLLDE(28,     0,     0,     696000000U),
PLLDE(58,     0,     1,     708000000U),
PLLDE(29,     0,     0,     720000000U),
PLLDE(60,     0,     1,     732000000U),
PLLDE(30,     0,     0,     744000000U),
PLLDE(62,     0,     1,     756000000U),
PLLDE(31,     0,     0,     768000000U),
PLLDE(64,     0,     1,     780000000U),
PLLDE(32,     0,     0,     792000000U),
PLLDE(66,     0,     1,     804000000U),
PLLDE(33,     0,     0,     816000000U),
PLLDE(68,     0,     1,     828000000U),
PLLDE(34,     0,     0,     840000000U),
PLLDE(70,     0,     1,     852000000U),
PLLDE(35,     0,     0,     864000000U),
PLLDE(72,     0,     1,     876000000U),
PLLDE(36,     0,     0,     888000000U),
PLLDE(74,     0,     1,     900000000U),
PLLDE(37,     0,     0,     912000000U),
PLLDE(76,     0,     1,     924000000U),
PLLDE(38,     0,     0,     936000000U),
PLLDE(78,     0,     1,     948000000U),
PLLDE(39,     0,     0,     960000000U),
PLLDE(80,     0,     1,     972000000U),
PLLDE(40,     0,     0,     984000000U),
PLLDE(82,     0,     1,     996000000U),
PLLDE(41,     0,     0,     1008000000U),
PLLDE(84,     0,     1,     1020000000U),
PLLDE(42,     0,     0,     1032000000U),
PLLDE(86,     0,     1,     1044000000U),
PLLDE(43,     0,     0,     1056000000U),
PLLDE(88,     0,     1,     1068000000U),
PLLDE(44,     0,     0,     1080000000U),
};

static unsigned int pllcpu_max, pllddr_max, pllperiph0_max,
		    pllperiph1_max, pllgpu_max, pllvideo0_max, pllvideo1_max,
		    pllve_max, pllde_max;

#define PLL_MAX_ASSIGN(name) (pll##name##_max = \
	factor_pll##name##_tbl[ARRAY_SIZE(factor_pll##name##_tbl)-1].freq)

void sunxi_clk_factor_initlimits(void)
{
	PLL_MAX_ASSIGN(cpu);
	PLL_MAX_ASSIGN(ddr);
	PLL_MAX_ASSIGN(periph0);
	PLL_MAX_ASSIGN(periph1);
	PLL_MAX_ASSIGN(gpu);
	PLL_MAX_ASSIGN(video0);
	PLL_MAX_ASSIGN(video1);
	PLL_MAX_ASSIGN(ve);
	PLL_MAX_ASSIGN(de);
}
